<?php

namespace UltraCommunity\Repository;


use UltraCommunity\Controllers\PostTypeController;
use UltraCommunity\Entities\ActivityEntity;
use UltraCommunity\Entities\ActivityMetaData;
use UltraCommunity\MchLib\Utils\MchUtils;
use UltraCommunity\MchLib\Utils\MchValidator;
use UltraCommunity\MchLib\WordPress\Repository\WpPostRepository;

final class ActivityRepository extends BaseRepository
{
	public static function getActivityByKey($activityKey)
	{
		$activityId = parent::getActivityIdFromKey($activityKey);
		if(empty($activityId))
			return null;

		$sqlQuery = "SELECT * FROM " . self::getActivityTableName() . " WHERE ActivityId = %d";
		$activityEntity = new ActivityEntity();
		foreach ( self::executePreparedQuery( self::getDbObject()->prepare( $sqlQuery, array($activityId)) )  as $entityInfo ) {
			foreach (get_object_vars($entityInfo) as $propertyName => $propertyValue){
				$activityEntity->{$propertyName} = $propertyValue;
			}
			break;
		}

		if(empty($activityEntity->ActivityId)){
			return null;
		}
		
		$activityEntity->ActivityId   = (int)$activityEntity->ActivityId;
		$activityEntity->PostTypeId   = (int)$activityEntity->PostTypeId;
		$activityEntity->TargetId     = (int)$activityEntity->TargetId;
		$activityEntity->UserId       = (int)$activityEntity->UserId;
		$activityEntity->StatusId     = (int)$activityEntity->StatusId;
		$activityEntity->ActionTypeId = (int)$activityEntity->ActionTypeId;
		$activityEntity->TargetTypeId = (int)$activityEntity->TargetTypeId;
		
		$activityEntity->MetaData = empty($activityEntity->MetaData) ? new ActivityMetaData() : \maybe_unserialize($activityEntity->MetaData);
		return $activityEntity;
	}



	public static function saveActivity(ActivityEntity $activityEntity)
	{
		if(empty($activityEntity->PostTypeId)) {
			$activityEntity->PostTypeId = PostTypeController::publishPostType(PostTypeController::getPostTypeInstance(PostTypeController::POST_TYPE_ACTIVITY));
		}
		
		empty($activityEntity->MetaData->Attachments) ?: $activityEntity->MetaData->Attachments = array_values($activityEntity->MetaData->Attachments);
		if(empty($activityEntity->MetaData->PostFormat) || $activityEntity->MetaData->PostFormat == ActivityEntity::ACTIVITY_POST_FORMAT_STATUS){
			$activityEntity->MetaData->PostFormat = null;
		}

		if(MchUtils::isNullOrEmpty(get_object_vars(MchUtils::filterObjectEmptyProperties($activityEntity->MetaData)))){
			$activityEntity->MetaData = null;
		}

		$activityEntity->MetaData = maybe_serialize($activityEntity->MetaData);

		!empty($activityEntity->CreatedDate) ?: $activityEntity->CreatedDate = current_time( 'mysql' );
		!empty($activityEntity->StatusId)    ?: $activityEntity->StatusId    = ActivityEntity::ACTIVITY_STATUS_ACTIVE;


		if(empty($activityEntity->ActivityId))
		{
			return (false === self::getDbObject()->insert(self::getActivityTableName(), array_filter((array)$activityEntity))) ? 0 : self::getDbObject()->insert_id;
		}

		$sqlQuery  = ' UPDATE ' . self::getActivityTableName()  . ' SET';
		$sqlQuery .= ' TargetTypeId  = ' . self::getDbObject()->prepare( '%d', $activityEntity->TargetTypeId );
		$sqlQuery .= ' ,ActionTypeId = ' . self::getDbObject()->prepare( '%d', $activityEntity->ActionTypeId );
		$sqlQuery .= ' ,UserId       = ' . self::getDbObject()->prepare( '%d', $activityEntity->UserId );
		$sqlQuery .= ' ,StatusId     = ' . self::getDbObject()->prepare( '%d', $activityEntity->StatusId );

		$sqlQuery .= ' ,CreatedDate  = ' . self::getDbObject()->prepare( '%s', $activityEntity->CreatedDate );

		$sqlQuery .= ' ,TargetId   = '  . (  MchValidator::isInteger($activityEntity->TargetId) ? self::getDbObject()->prepare( '%d', $activityEntity->TargetId ) : 'null' );
		$sqlQuery .= ' ,PostTypeId = '  . (  MchValidator::isInteger($activityEntity->PostTypeId) ? self::getDbObject()->prepare( '%d', $activityEntity->PostTypeId ) : 'null' );
		$sqlQuery .= ' ,MetaData   = '  . ( !empty($activityEntity->MetaData ) ? self::getDbObject()->prepare( '%s', $activityEntity->MetaData ) : 'null' );

		$sqlQuery .= ' WHERE ActivityId = %d';

		return self::executePreparedQuery( self::getDbObject()->prepare( $sqlQuery, array($activityEntity->ActivityId)) );

	}
	
	
	/**
	 * @param ActivityEntity $activityEntity
	 * @param int            $pageNumber
	 * @param int            $recordsPerPage
	 * @param array          $arrAdditionalParams
	 *
	 * @return ActivityEntity[]
	 */
	public static function findByEntityProperties(ActivityEntity $activityEntity, $pageNumber = 1, $recordsPerPage = 10, $arrAdditionalParams = array())
	{
		$arrProperties = empty($arrAdditionalParams) ? get_object_vars($activityEntity) : array_merge( get_object_vars($activityEntity), $arrAdditionalParams );
		$arrProperties = array_filter($arrProperties, 'is_scalar');
		
		( ( $pageNumber = (int) $pageNumber ) > 0 ) ?: $pageNumber = 1;
		
		$selectStatement  = 'SELECT * FROM ' . self::getActivityTableName() . ' WHERE ';
		$selectStatement .= implode('=%s AND ', array_keys($arrProperties)) . '= %s ';
		
		$selectStatement .= 'ORDER BY ActivityId DESC LIMIT %d, %d';
		
		$arrParams = array_values($arrProperties);
		$arrParams[] = ( $pageNumber - 1 ) * $recordsPerPage;
		$arrParams[] = $recordsPerPage;
		
		
		$arrEntities = array();
		foreach (self::executePreparedQuery(self::getDbObject()->prepare($selectStatement, $arrParams)) as $resultEntity)
		{
			$activityEntity = new ActivityEntity();
			foreach ((array)$resultEntity as $property => $value) {
				$activityEntity->{$property} = $value;
			}
			
			$activityEntity->ActivityId   = (int)$activityEntity->ActivityId;
			$activityEntity->PostTypeId   = (int)$activityEntity->PostTypeId;
			$activityEntity->TargetId     = (int)$activityEntity->TargetId;
			$activityEntity->UserId       = (int)$activityEntity->UserId;
			$activityEntity->StatusId     = (int)$activityEntity->StatusId;
			$activityEntity->ActionTypeId = (int)$activityEntity->ActionTypeId;
			$activityEntity->TargetTypeId = (int)$activityEntity->TargetTypeId;
			
			$activityEntity->MetaData = empty($activityEntity->MetaData) ? new ActivityMetaData() : \maybe_unserialize($activityEntity->MetaData);
			$arrEntities[] = $activityEntity;
		}

		return $arrEntities;

	}

	public static function countActivityByEntityProperties(ActivityEntity $activityEntity)
	{
		$arrProperties = \array_filter(\get_object_vars($activityEntity), 'is_scalar');
		
		$selectStatement  = 'SELECT COUNT(*) FROM ' . self::getActivityTableName() . ' WHERE UserId > 0 AND ';
		$selectStatement .= implode('=%s AND ', \array_keys($arrProperties)) . '=%s ';

		$arrParams = array_values($arrProperties);
		
		return (int)self::getDbObject()->get_var(self::getDbObject()->prepare($selectStatement, $arrParams));
		
	}
	
	public static function deleteActivity($activityKey)
	{
		if(null === ($activityEntity = self::getActivityByKey($activityKey)))
			return;
		
		$sqlQuery = "DELETE FROM " . self::getActivityTableName() . " WHERE ActivityId = %d";
		
		self::executePreparedQuery( self::getDbObject()->prepare( $sqlQuery, array($activityEntity->ActivityId)) );
		
		if(null === ($wpPost = WpPostRepository::findByPostId($activityEntity->PostTypeId)) || empty($wpPost->post_type))
			return;
		
		if($wpPost->post_type !== PostTypeController::POST_TYPE_ACTIVITY)
			return;
		
		WpPostRepository::delete($wpPost->ID);
	
	}
	
	public static function createActivityTable()
	{
		if ( parent::tableExists( self::getActivityTableName() ) ) {
			return false;
		}

		global $wpdb;

		function_exists('dbDelta') || require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );

		$createTableStatement = "CREATE TABLE " . self::getActivityTableName() . " (
                                                                  ActivityId int unsigned NOT NULL auto_increment,
                                                                  TargetTypeId tinyint unsigned NOT NULL,
                                                                  ActionTypeId tinyint unsigned NOT NULL,
                                                                  UserId  int unsigned NOT NULL,
                                                                  TargetId int unsigned NULL,
                                                                  PostTypeId int unsigned NULL,
                                                                  CreatedDate TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,
                                                                  StatusId tinyint unsigned NULL,
                                                                  MetaData mediumtext NULL DEFAULT NULL,
                                                                  PRIMARY KEY  (ActivityId),
                                                                  INDEX idx_uc_activity (UserId, TargetId, PostTypeId, StatusId)
                                                                )";

		$createTableStatement .= ! empty( $wpdb->charset ) ? " DEFAULT CHARACTER SET {$wpdb->charset}" : '';
		$createTableStatement .= ! empty( $wpdb->collate ) ? " COLLATE {$wpdb->collate}" : '';

		$result = dbDelta( $createTableStatement );

		return ! empty( $result ) ? true : false;
	}




}