<?php

namespace UltraCommunity\FrontPages;

use UltraCommunity\Controllers\FrontPageController;
use UltraCommunity\Controllers\GroupController;
use UltraCommunity\Controllers\ModulesController;
use UltraCommunity\Controllers\TemplatesController;
use UltraCommunity\Controllers\UserRelationsController;
use UltraCommunity\Controllers\UserRoleController;
use UltraCommunity\Entities\GroupEntity;
use UltraCommunity\MchLib\Utils\MchMinifier;
use UltraCommunity\MchLib\Utils\MchUtils;
use UltraCommunity\MchLib\Utils\MchValidator;
use UltraCommunity\MchLib\Utils\MchWpUtils;
use UltraCommunity\Controllers\UserController;
use UltraCommunity\Modules\Forms\FormFields\BaseField;
use UltraCommunity\Modules\Forms\FormFields\ProfileSectionField;
use UltraCommunity\UltraCommHelper;
use UltraCommunity\UltraCommHooks;

class UserSettingsPage extends BasePage
{
	CONST SETTINGS_SECTION_PROFILE_AVATAR_PICTURE = 'profile-picture';
	CONST SETTINGS_SECTION_PROFILE_COVER_PICTURE  = 'profile-cover';
	CONST SETTINGS_SECTION_PROFILE_FORM_SECTION   = 'profile-form-section';

	CONST SETTINGS_SECTION_GROUPS_CREATE_GROUP  = 'create-group';
	CONST SETTINGS_SECTION_GROUPS_EDIT_GROUP    = 'edit-group';
	CONST SETTINGS_SECTION_GROUPS_GROUP_DELETE  = 'delete-group';
	CONST SETTINGS_SECTION_GROUPS_GROUP_PICTURE = 'picture-group';
	CONST SETTINGS_SECTION_GROUPS_GROUP_COVER   = 'cover-group';
	CONST SETTINGS_SECTION_GROUPS_GROUP_MEMBERS = 'members-group';

	CONST SETTINGS_SECTION_FRIENDS_REQUESTS     = 'friend-requests';
	CONST SETTINGS_SECTION_FRIENDS_MY_FRIENDS   = 'my-friends';
	CONST SETTINGS_SECTION_FOLLOW_FOLLOWING     = 'following';
	CONST SETTINGS_SECTION_FOLLOW_FOLLOWERS     = 'my-followers';

	CONST SETTINGS_SECTION_GROUPS_MY_GROUPS     = 'my-groups';
	CONST SETTINGS_SECTION_GROUPS_JOINED_GROUPS = 'joined-groups';

	CONST SETTINGS_SECTION_ACCOUNT_CHANGE_PASSWORD   = 'change-password';
	CONST SETTINGS_SECTION_ACCOUNT_DELETE            = 'delete-account';


	private $activeSection = null;
	private $activeSectionResourceIdentifier = null;

	private $activeSectionPageNumber = 0;

	public function __construct($pageId = null)
	{
		parent::__construct($pageId);
	}

	public static function getAllSettingsSections()
	{
		return \apply_filters(UltraCommHooks::FILTER_USER_PROFILE_SETTINGS_SECTIONS, array_merge(self::getProfileSettingsSections(), self::getGroupsSettingsSections(), self::getUserRelationsSettingsSections()));
	}

	public static function getProfileSettingsSections($justKeys = false)
	{
		$arrProfileSections =   array(

				self::SETTINGS_SECTION_PROFILE_COVER_PICTURE   => false,
				self::SETTINGS_SECTION_PROFILE_AVATAR_PICTURE  => false,
				self::SETTINGS_SECTION_ACCOUNT_CHANGE_PASSWORD => false,
				self::SETTINGS_SECTION_ACCOUNT_DELETE          => false,
				self::SETTINGS_SECTION_PROFILE_FORM_SECTION    => true,

		);

		return $justKeys ? array_keys($arrProfileSections) : $arrProfileSections;

	}

	public static function getGroupsSettingsSections($justKeys = false)
	{

		$arrGroupsSections =   array(
				self::SETTINGS_SECTION_GROUPS_CREATE_GROUP  => false,
				self::SETTINGS_SECTION_GROUPS_MY_GROUPS     => false,
				self::SETTINGS_SECTION_GROUPS_JOINED_GROUPS => false,
				self::SETTINGS_SECTION_GROUPS_EDIT_GROUP    => true,
				self::SETTINGS_SECTION_GROUPS_GROUP_PICTURE => true,
				self::SETTINGS_SECTION_GROUPS_GROUP_COVER   => true,
				self::SETTINGS_SECTION_GROUPS_GROUP_MEMBERS => true,
				self::SETTINGS_SECTION_GROUPS_GROUP_DELETE  => true,

		);

		return $justKeys ? array_keys($arrGroupsSections) : $arrGroupsSections;
	}

	public static function getUserRelationsSettingsSections($justKeys = false)
	{

		$arrUserRelationSections =   array(
			self::SETTINGS_SECTION_FRIENDS_REQUESTS   => false,
			self::SETTINGS_SECTION_FRIENDS_MY_FRIENDS => false,
			self::SETTINGS_SECTION_FOLLOW_FOLLOWING   => false,
			self::SETTINGS_SECTION_FOLLOW_FOLLOWERS   => false,

		);

		return $justKeys ? array_keys($arrUserRelationSections) : $arrUserRelationSections;


	}


	public function getSectionDescription($sectionKey)
	{
		$sectionName = null;
		switch ($sectionKey)
		{
			case self::SETTINGS_SECTION_PROFILE_AVATAR_PICTURE  : $sectionName =  esc_html__('Profile Picture', 'ultra-community');break;
			case self::SETTINGS_SECTION_PROFILE_COVER_PICTURE   : $sectionName =  esc_html__('Profile Cover', 'ultra-community');break;
			case self::SETTINGS_SECTION_ACCOUNT_CHANGE_PASSWORD : $sectionName =  esc_html__('Change Password', 'ultra-community');break;
			case self::SETTINGS_SECTION_ACCOUNT_DELETE          : $sectionName =  esc_html__('Delete Account', 'ultra-community');break;

			case self::SETTINGS_SECTION_GROUPS_MY_GROUPS        : $sectionName =  esc_html__('My Groups', 'ultra-community');break;
			case self::SETTINGS_SECTION_GROUPS_CREATE_GROUP     : $sectionName =  esc_html__('Create Group', 'ultra-community');break;
			case self::SETTINGS_SECTION_GROUPS_JOINED_GROUPS    : $sectionName =  esc_html__('Joined Groups', 'ultra-community');break;

			case self::SETTINGS_SECTION_GROUPS_EDIT_GROUP       : $sectionName =  esc_html__('Settings', 'ultra-community');break;
			case self::SETTINGS_SECTION_GROUPS_GROUP_PICTURE    : $sectionName =  esc_html__('Picture', 'ultra-community');break;
			case self::SETTINGS_SECTION_GROUPS_GROUP_COVER      : $sectionName =  esc_html__('Cover', 'ultra-community');break;
			case self::SETTINGS_SECTION_GROUPS_GROUP_MEMBERS    : $sectionName =  esc_html__('Members', 'ultra-community');break;
			case self::SETTINGS_SECTION_GROUPS_GROUP_DELETE     : $sectionName =  esc_html__('Delete', 'ultra-community');break;


			case self::SETTINGS_SECTION_FRIENDS_REQUESTS   : $sectionName =  esc_html__('Friend Requests', 'ultra-community');break;
			case self::SETTINGS_SECTION_FRIENDS_MY_FRIENDS : $sectionName =  esc_html__('My Friends', 'ultra-community');break;
			case self::SETTINGS_SECTION_FOLLOW_FOLLOWING   : $sectionName =  esc_html__('Following', 'ultra-community');break;
			case self::SETTINGS_SECTION_FOLLOW_FOLLOWERS   : $sectionName =  esc_html__('Followers', 'ultra-community');break;


		}

		return esc_html(apply_filters(UltraCommHooks::FILTER_USER_PROFILE_SETTINGS_SECTION_NAME, $sectionName, $sectionKey));

	}

	private function getSectionSubMenuIcon($sectionKey)
	{
		switch($sectionKey)
		{
			case self::SETTINGS_SECTION_GROUPS_EDIT_GROUP       : return 'fa-cog';
			case self::SETTINGS_SECTION_GROUPS_GROUP_PICTURE    : return 'fa-user-circle-o';
			case self::SETTINGS_SECTION_GROUPS_GROUP_COVER      : return 'fa-image';
			case self::SETTINGS_SECTION_GROUPS_GROUP_MEMBERS    : return 'fa-users';
			case self::SETTINGS_SECTION_GROUPS_GROUP_DELETE     : return 'fa-trash';

		}

		return null;
	}

	private static function isGroupSection($sectionKey)
	{
		$arrGroupsSections = self::getGroupsSettingsSections();
		return isset($arrGroupsSections[$sectionKey]);
	}

	private static function isProfileSection($sectionKey)
	{
		$arrProfileSections = self::getProfileSettingsSections();
		return isset($arrProfileSections[$sectionKey]);
	}

	public static function sectionRequiresIdentifierInUrl($sectionKey)
	{
		$arrAllSections = self::getAllSettingsSections();
		return !empty($arrAllSections[$sectionKey]);
	}

	public function getShortCodeContent($arrAttributes, $shortCodeText = null, $shortCodeName = null)
	{
		return null;
	}

	public function processRequest()
	{

		UserController::currentUserCanEditProfile() ?: FrontPageController::redirectToLogInPage();

		if($this->getActiveSection() === self::SETTINGS_SECTION_PROFILE_FORM_SECTION)
		{
			if(null === $this->getActiveSectionResourceIdentifier())
			{
				foreach(self::getUserProfileFormFields() as $profileFormField)
				{
					if( ! $profileFormField instanceof ProfileSectionField )
						continue;
					$this->setActiveSectionResourceIdentifier($profileFormField->UniqueId);
					break;
				}
			}

			if(null === $this->getActiveSectionResourceIdentifier()) // the form is empty
			{
				$this->setActiveSection(self::SETTINGS_SECTION_PROFILE_AVATAR_PICTURE);
			}

		}

		if(self::sectionRequiresIdentifierInUrl($this->getActiveSection()))
		{
			if(MchUtils::isNullOrEmpty($this->getActiveSectionResourceIdentifier())){
				MchWpUtils::redirectTo404();
			}

			if(self::isGroupSection($this->getActiveSection()))
			{
				if(!GroupController::userCanEditGroup(UserController::getLoggedInUser(), $this->getActiveSectionResourceIdentifier())){
					MchWpUtils::redirectTo404();
				}

				GroupController::setProfiledGroup($this->getActiveSectionResourceIdentifier());
			}
		}

		MchWpUtils::addFilterHook(UltraCommHooks::FILTER_PAGE_SIDEBAR_HTML_CLASSES, function($arrSideBarClasses){$arrSideBarClasses[] = 'uch-left-sidebar';return $arrSideBarClasses;});
		MchWpUtils::addFilterHook(UltraCommHooks::FILTER_PAGE_CONTENT_HTML_CLASSES, function($arrSideBarClasses){$arrSideBarClasses[] = 'uc-user-settings-page-content';return $arrSideBarClasses;});

	}


	public function setActiveSection($activeSection)
	{
		$this->activeSection = $activeSection;
	}

	public function getActiveSection()
	{
		return $this->activeSection;
	}

	public function getActiveSectionSlug()
	{
		return $this->getActiveSection();
	}

	public function getActiveSectionResourceIdentifier()
	{
		return $this->activeSectionResourceIdentifier;
	}

	public function setActiveSectionResourceIdentifier($resourceIdentifier)
	{
		$resourceIdentifier = trim($resourceIdentifier);
		$this->activeSectionResourceIdentifier = empty($resourceIdentifier) ? null : $resourceIdentifier;
	}

	public function setActiveSectionPageNumber($pageNumber)
	{
		$this->activeSectionPageNumber = (MchValidator::isInteger($pageNumber) && $pageNumber > 0)  ? (int)$pageNumber : 0;
	}


	public function getActiveSectionPageNumber()
	{
		return $this->activeSectionPageNumber;
	}

	public function isAuthenticationRequired()
	{
		return true;
	}


	protected function getPageHiddenContent()
	{
		$outputContent = '';

//		if($this->getActiveSection() === self::SETTINGS_SECTION_GROUPS_MY_GROUPS || $this->getActiveSection() === self::SETTINGS_SECTION_GROUPS_JOINED_GROUPS)
//		{
//			$popupTitle   = esc_html__('Delete Group', 'ultra-community');
//			$popupContent = esc_html__('Are you sure you want to delete this group?', 'ultra-community');
//
//			$outputContent .= $this->getConfirmationPopupContent($popupTitle, $popupContent, 'uc-confirmation-popup-delete-group');
//
//			$popupTitle   = esc_html__('Leave Group', 'ultra-community');
//			$popupContent = esc_html__('Are you sure you want to leave this group?', 'ultra-community');
//
//			$outputContent .= $this->getConfirmationPopupContent($popupTitle, $popupContent, 'uc-confirmation-popup-leave-group');
//
//
//		}

		return $outputContent . parent::getPageHiddenContent();
	}


	public function getPageCustomCss()
	{
		return  parent::getPageCustomCss();
	}

	public function getHeaderMarkup() {
		return null;
	}

	public function getNavBarMarkup() {
		return null;
	}

	public function getSideBarMarkup()
	{

		$sideBarProfileSettingsItems = array();
		foreach (self::getProfileSettingsSections(true) as $profileSettingsSection)
		{

			switch ($profileSettingsSection)
			{
				case self::SETTINGS_SECTION_PROFILE_AVATAR_PICTURE :

					$profileSettingsItem = new \stdClass();
					$profileSettingsItem->Icon     = 'fa-camera';
					$profileSettingsItem->Title    = self::getSectionDescription($profileSettingsSection);
					$profileSettingsItem->Url      = FrontPageController::getUserSettingsPageUrlBySection($profileSettingsSection, UserController::getProfiledUser()->NiceName);
					$profileSettingsItem->IsActive = ($this->getActiveSection() === $profileSettingsSection);

					if(MchWpUtils::isAdminLoggedIn() || UserRoleController::currentUserCanChangeProfileAvatar()){
						$sideBarProfileSettingsItems[] = $profileSettingsItem;
					}


					break;


				case self::SETTINGS_SECTION_PROFILE_COVER_PICTURE :

					$profileSettingsItem = new \stdClass();
					$profileSettingsItem->Icon     = 'fa-picture-o';
					$profileSettingsItem->Title    = self::getSectionDescription($profileSettingsSection);
					$profileSettingsItem->Url      = FrontPageController::getUserSettingsPageUrlBySection($profileSettingsSection, UserController::getProfiledUser()->NiceName);
					$profileSettingsItem->IsActive = ($this->getActiveSection() === $profileSettingsSection);

					if(MchWpUtils::isAdminLoggedIn() || UserRoleController::currentUserCanChangeProfileCover()){
						$sideBarProfileSettingsItems[] = $profileSettingsItem;
					}

					break;


				case self::SETTINGS_SECTION_PROFILE_FORM_SECTION :

					$arrUserProfileFields = UserController::currentUserCanEditProfile() ? self::getUserProfileFormFields() : array();

					foreach ($arrUserProfileFields as $userProfileField)
					{
						if(! $userProfileField instanceof ProfileSectionField)
							continue;

						$profileSettingsItem = new \stdClass();

						$profileSettingsItem->Title = esc_html($userProfileField->Title);
						$profileSettingsItem->Url   = FrontPageController::getUserSettingsPageUrlBySection($profileSettingsSection, UserController::getProfiledUser()->NiceName, $userProfileField->UniqueId);
						$profileSettingsItem->Icon  = 'fa-user-circle-o';

						$profileSettingsItem->IsActive = ($this->getActiveSectionResourceIdentifier() === $userProfileField->UniqueId);


						$sideBarProfileSettingsItems[] = $profileSettingsItem;
					}

					break;
			}

		}

		$sideBarGroupsSettingsItems = array();
		if(UserRoleController::currentUserCanCreateUserGroups())
		{
			$groupsSettingsItem        = new \stdClass();
			$groupsSettingsItem->Url   = FrontPageController::getUserSettingsPageUrlBySection(self::SETTINGS_SECTION_GROUPS_CREATE_GROUP, UserController::getProfiledUser()->NiceName);;
			$groupsSettingsItem->Title = self::getSectionDescription(self::SETTINGS_SECTION_GROUPS_CREATE_GROUP);
			$groupsSettingsItem->Icon = 'fa-plus';

			$groupsSettingsItem->IsActive = $this->getActiveSection() === self::SETTINGS_SECTION_GROUPS_CREATE_GROUP;

			$sideBarGroupsSettingsItems[] = $groupsSettingsItem;

		}


		if(! MchUtils::isNullOrEmpty(GroupController::getUserCreatedGroups(UserController::getProfiledUser())))
		{
			$groupsSettingsItem        = new \stdClass();
			$groupsSettingsItem->Url   = FrontPageController::getUserSettingsPageUrlBySection(self::SETTINGS_SECTION_GROUPS_MY_GROUPS, UserController::getProfiledUser()->NiceName);;
			$groupsSettingsItem->Title = self::getSectionDescription(self::SETTINGS_SECTION_GROUPS_MY_GROUPS);
			$groupsSettingsItem->Icon = 'fa-th-list';

			$groupsSettingsItem->IsActive = $this->getActiveSection() === self::SETTINGS_SECTION_GROUPS_MY_GROUPS;

			$sideBarGroupsSettingsItems[] = $groupsSettingsItem;

		}

		if(! MchUtils::isNullOrEmpty(GroupController::getUserJoinedGroups(UserController::getProfiledUser())))
		{
			$groupsSettingsItem        = new \stdClass();
			$groupsSettingsItem->Url   = FrontPageController::getUserSettingsPageUrlBySection(self::SETTINGS_SECTION_GROUPS_JOINED_GROUPS, UserController::getProfiledUser()->NiceName);;
			$groupsSettingsItem->Title = self::getSectionDescription(self::SETTINGS_SECTION_GROUPS_JOINED_GROUPS);
			$groupsSettingsItem->Icon = 'fa-list-ul';

			$groupsSettingsItem->IsActive = $this->getActiveSection() === self::SETTINGS_SECTION_GROUPS_JOINED_GROUPS;

			$sideBarGroupsSettingsItems[] = $groupsSettingsItem;

		}

		$sideBarAccountSettingsItems = array();
		$sideBarAccountSettingsItems[self::SETTINGS_SECTION_ACCOUNT_CHANGE_PASSWORD] = 	FrontPageController::getUserSettingsPageUrlBySection(self::SETTINGS_SECTION_ACCOUNT_CHANGE_PASSWORD, UserController::getProfiledUser()->NiceName);
		$sideBarAccountSettingsItems[self::SETTINGS_SECTION_ACCOUNT_DELETE]          = 	FrontPageController::getUserSettingsPageUrlBySection(self::SETTINGS_SECTION_ACCOUNT_DELETE, UserController::getProfiledUser()->NiceName);


		if(MchWpUtils::isAdminUser(UserController::getProfiledUser()->Id)) // No Delete account OR Change Password for Administrators
		{
			unset($sideBarAccountSettingsItems[self::SETTINGS_SECTION_ACCOUNT_CHANGE_PASSWORD]);
			unset($sideBarAccountSettingsItems[self::SETTINGS_SECTION_ACCOUNT_DELETE]);
		}

		if( ! MchWpUtils::isAdminUser(UserController::getProfiledUser()->Id) )
		{
			if(!MchWpUtils::isAdminLoggedIn() && !UserRoleController::currentUserCanChangePassword()){
				unset($sideBarAccountSettingsItems[self::SETTINGS_SECTION_ACCOUNT_CHANGE_PASSWORD]);
			}

			if(!MchWpUtils::isAdminLoggedIn() && !UserRoleController::currentUserCanDeleteAccount()){
				unset($sideBarAccountSettingsItems[self::SETTINGS_SECTION_ACCOUNT_DELETE]);
			}

		}

		foreach($sideBarAccountSettingsItems as $accountSectionKey => $sectionUrl)
		{
			$accountSettingsItem = new \stdClass();
			$accountSettingsItem->Url   = $sectionUrl;

			$accountSettingsItem->IsActive = ($this->getActiveSection() === $accountSectionKey);

			switch($accountSectionKey)
			{
				case self::SETTINGS_SECTION_ACCOUNT_CHANGE_PASSWORD : $accountSettingsItem->Title = self::getSectionDescription($accountSectionKey); $accountSettingsItem->Icon = 'fa-lock'; break;
				case self::SETTINGS_SECTION_ACCOUNT_DELETE          : $accountSettingsItem->Title = self::getSectionDescription($accountSectionKey);  $accountSettingsItem->Icon = 'fa-trash'; break;
			}

			$sideBarAccountSettingsItems[$accountSectionKey] = $accountSettingsItem;
		}

		$sideBarUserRelationsItems = array();
		if(ModulesController::isModuleRegistered(ModulesController::MODULE_USER_FRIENDS))
		{
			$sideBarUserRelationsItems[self::SETTINGS_SECTION_FRIENDS_MY_FRIENDS] = FrontPageController::getUserSettingsPageUrlBySection(self::SETTINGS_SECTION_FRIENDS_MY_FRIENDS, UserController::getProfiledUser()->NiceName);
			$sideBarUserRelationsItems[self::SETTINGS_SECTION_FRIENDS_REQUESTS]   = FrontPageController::getUserSettingsPageUrlBySection(self::SETTINGS_SECTION_FRIENDS_REQUESTS, UserController::getProfiledUser()->NiceName);
		}

		if(ModulesController::isModuleRegistered(ModulesController::MODULE_USER_FOLLOWERS))
		{
			$sideBarUserRelationsItems[self::SETTINGS_SECTION_FOLLOW_FOLLOWERS] = FrontPageController::getUserSettingsPageUrlBySection(self::SETTINGS_SECTION_FOLLOW_FOLLOWERS, UserController::getProfiledUser()->NiceName);
			$sideBarUserRelationsItems[self::SETTINGS_SECTION_FOLLOW_FOLLOWING] = FrontPageController::getUserSettingsPageUrlBySection(self::SETTINGS_SECTION_FOLLOW_FOLLOWING, UserController::getProfiledUser()->NiceName);
		}

		foreach($sideBarUserRelationsItems as $accountSectionKey => $sectionUrl)
		{
			$relationItem = new \stdClass();
			$relationItem->Url   = $sectionUrl;
			$relationItem->Title = self::getSectionDescription($accountSectionKey);
			$relationItem->Icon = 'fa-user-circle-o';

			$relationItem->IsActive = ($this->getActiveSection() === $accountSectionKey);


			$sideBarUserRelationsItems[$accountSectionKey] = $relationItem;
		}

		$arrSideBarArguments = array(
			'sideBarProfileSettingsItems' => $sideBarProfileSettingsItems,
			'sideBarAccountSettingsItems' => $sideBarAccountSettingsItems,
			'sideBarGroupsSettingsItems'  => $sideBarGroupsSettingsItems,
			'sideBarUserRelationsItems'   => $sideBarUserRelationsItems,
		);


		$arrSideBarArguments = (array)apply_filters(UltraCommHooks::FILTER_USER_PROFILE_SETTINGS_SIDEBAR_ITEMS, $arrSideBarArguments, $this);


		echo TemplatesController::getTemplateOutputContent('user-settings-sidebar', $arrSideBarArguments);

	}

	public function getContentMarkup()
	{
		$templateName         = null;
		$arrTemplateArguments = array();

		switch ($this->getActiveSection())
		{
			case self::SETTINGS_SECTION_PROFILE_AVATAR_PICTURE :
				$templateName = 'user-settings-profile-picture'; break;

			case self::SETTINGS_SECTION_PROFILE_COVER_PICTURE :
				$templateName = 'user-settings-profile-cover'; break;

			case self::SETTINGS_SECTION_PROFILE_FORM_SECTION :
				$templateName = 'user-settings-profile-section'; break;

			case self::SETTINGS_SECTION_ACCOUNT_CHANGE_PASSWORD :
				$templateName = 'user-settings-account-change-password'; break;

			case self::SETTINGS_SECTION_ACCOUNT_DELETE :
				$templateName = 'user-settings-account-delete'; break;

			case self::SETTINGS_SECTION_GROUPS_CREATE_GROUP :
			case self::SETTINGS_SECTION_GROUPS_EDIT_GROUP :
				$templateName = 'user-settings-groups-edit-group'; break;

			case self::SETTINGS_SECTION_GROUPS_MY_GROUPS :
				$templateName = 'user-settings-groups-my-groups'; break;

			case self::SETTINGS_SECTION_GROUPS_JOINED_GROUPS :
				$templateName = 'user-settings-groups-joined-groups'; break;

			case self::SETTINGS_SECTION_GROUPS_GROUP_COVER :
				$templateName = 'user-settings-groups-group-cover'; break;

			case self::SETTINGS_SECTION_GROUPS_GROUP_PICTURE :
				$templateName = 'user-settings-groups-group-picture'; break;

			case self::SETTINGS_SECTION_GROUPS_GROUP_MEMBERS :
				$templateName = 'user-settings-groups-group-members'; break;

			case self::SETTINGS_SECTION_GROUPS_GROUP_DELETE :
				$templateName = 'user-settings-groups-group-delete'; break;

			case self::SETTINGS_SECTION_FRIENDS_MY_FRIENDS  :
				$templateName = TemplatesController::USER_PROFILE_FRIENDS_TEMPLATE; $arrTemplateArguments['primaryUserKey'] = UserController::getProfiledUser(); $arrTemplateArguments['currentPage'] = $this->getActiveSectionPageNumber(); break;

			case self::SETTINGS_SECTION_FRIENDS_REQUESTS  :
				$templateName = TemplatesController::USER_PROFILE_FRIEND_REQUESTS_TEMPLATE; $arrTemplateArguments['primaryUserKey'] = UserController::getProfiledUser(); $arrTemplateArguments['currentPage'] = $this->getActiveSectionPageNumber(); break;

			case self::SETTINGS_SECTION_FOLLOW_FOLLOWERS    :
				$templateName = TemplatesController::USER_PROFILE_FOLLOWERS_TEMPLATE; $arrTemplateArguments['primaryUserKey'] = UserController::getProfiledUser(); $arrTemplateArguments['currentPage'] = $this->getActiveSectionPageNumber(); break;

			case self::SETTINGS_SECTION_FOLLOW_FOLLOWING    :
				$templateName = TemplatesController::USER_PROFILE_FOLLOWING_TEMPLATE; $arrTemplateArguments['primaryUserKey'] = UserController::getProfiledUser(); $arrTemplateArguments['currentPage'] = $this->getActiveSectionPageNumber(); break;

		}

		if($this->getActiveSection() === self::SETTINGS_SECTION_PROFILE_FORM_SECTION)
		{
			$profileSection = null;
			foreach(self::getUserProfileFormFields() as $profileFormField)
			{
				if($profileFormField instanceof ProfileSectionField && $profileFormField->UniqueId == $this->getActiveSectionResourceIdentifier())
				{
					$profileSection = new \stdClass();
					$profileSection->Title = esc_html(MchWpUtils::stripHtmlTags($profileFormField->Title));
					$profileSection->Fields = array();
					continue;
				}

				if(null === $profileSection)
					continue;

				if($profileFormField instanceof ProfileSectionField)
					break;

				if(!UserController::currentUserCanViewProfileFormField($profileFormField, true))
					continue;

				$profileFormField->Value = UserController::getUserProfileFormFieldValue(UserController::getProfiledUser(), $profileFormField);


				if(!empty($profileFormField->OptionsList) && is_scalar($profileFormField->Value))
				{
					if(false !== ($savedOptionValue = array_search($profileFormField->Value, (array)$profileFormField->OptionsList))){
						$profileFormField->Value = $savedOptionValue;
					}

					unset($savedOptionValue);
				}


				!empty($profileFormField->Value) ?: $profileFormField->Value = MchWpUtils::stripSlashes(html_entity_decode(htmlspecialchars_decode($profileFormField->DefaultValue)));

				$profileSection->Fields[] = $profileFormField;

			}

			(null === $profileSection) ?:  $arrTemplateArguments['profileSection'] = $profileSection;
		}


		if($this->getActiveSection() === self::SETTINGS_SECTION_GROUPS_CREATE_GROUP)
		{
			$arrTemplateArguments['editableGroupEntity'] = new GroupEntity();
		}

		if($this->getActiveSection() === self::SETTINGS_SECTION_GROUPS_EDIT_GROUP)
		{
			$arrTemplateArguments['editableGroupEntity'] = GroupController::getGroupEntityBy($this->getActiveSectionResourceIdentifier());
		}

		if($this->getActiveSection() === self::SETTINGS_SECTION_GROUPS_GROUP_DELETE)
		{
			$arrTemplateArguments['editableGroupEntity'] = GroupController::getGroupEntityBy($this->getActiveSectionResourceIdentifier());
		}

		if($this->getActiveSection() === self::SETTINGS_SECTION_GROUPS_MY_GROUPS)
		{
			$arrGroupsList = array();
			foreach (GroupController::getUserCreatedGroups(UserController::getProfiledUser(), 1, PHP_INT_MAX) as $groupEntity)
			{

				if(!GroupController::userCanEditGroup(UserController::getLoggedInUser(), $groupEntity))
					continue;

				$groupEntity->escapeFields();

				$groupEntity->EditUrl = FrontPageController::getUserSettingsPageUrlBySection(self::SETTINGS_SECTION_GROUPS_EDIT_GROUP, UserController::getProfiledUser()->NiceName, $groupEntity->Id);
				$groupEntity->Url     = UltraCommHelper::getGroupUrl($groupEntity);

				$groupEntity->PictureUrl   = UltraCommHelper::getGroupPictureUrl($groupEntity);
				$groupEntity->MembersCount = GroupController::countGroupUsers($groupEntity->Id);
				$groupEntity->CreatedDate  = mysql2date('M j, Y', $groupEntity->CreatedDate);

				$arrGroupsList[] = $groupEntity;
			}

			$arrTemplateArguments['arrUserGroups'] = $arrGroupsList;
		}

		if($this->getActiveSection() === self::SETTINGS_SECTION_GROUPS_JOINED_GROUPS)
		{
			$arrGroupsList = array();
			foreach (GroupController::getUserJoinedGroups(UserController::getProfiledUser(), 1, PHP_INT_MAX) as $groupEntity)
			{
				$groupEntity->escapeFields();

				$groupEntity->Url     = UltraCommHelper::getGroupUrl($groupEntity);

				$groupEntity->PictureUrl   = UltraCommHelper::getGroupPictureUrl($groupEntity);
				$groupEntity->MembersCount = GroupController::countGroupUsers($groupEntity->Id);
				$groupEntity->CreatedDate  = mysql2date('M j, Y', $groupEntity->CreatedDate);

				$arrGroupsList[] = $groupEntity;
			}

			$arrTemplateArguments['arrUserJoinedGroups'] = $arrGroupsList;
		}


		if($this->getActiveSection() === self::SETTINGS_SECTION_GROUPS_GROUP_MEMBERS)
		{
			$arrTemplateArguments['groupKey']    = $this->getActiveSectionResourceIdentifier();
			$arrTemplateArguments['currentPage'] =  $this->getActiveSectionPageNumber();

		}


		echo TemplatesController::getTemplateOutputContent(TemplatesController::USER_PROFILE_NAV_BAR_TEMPLATE, $this->getSubMenuTemplateArguments()) . TemplatesController::getTemplateOutputContent($templateName, $arrTemplateArguments);
	}



	private function getSubMenuTemplateArguments()
	{
		$arrNavBarArguments = array(
				//'mainMenuAdditionalHtmlClasses' => 'uc-menu-stacked' ,
				'sectionsListsOutput' => null
		);



		if(self::isGroupSection($this->getActiveSection()) && $this->sectionRequiresIdentifierInUrl($this->getActiveSection()))
		{
			$arrNavBarArguments['mainMenuAdditionalHtmlClasses'] = 'uc-menu-stacked';

			foreach(self::getGroupsSettingsSections(true) as $sectionKey)
			{
				if(!$this->sectionRequiresIdentifierInUrl($sectionKey))
					continue;

				$sectionName = self::getSectionDescription($sectionKey);

				$sectionUrl  = FrontPageController::getUserSettingsPageUrlBySection($sectionKey, UserController::getProfiledUser()->NiceName, $this->getActiveSectionResourceIdentifier());

				$iconClass = $this->getSectionSubMenuIcon($sectionKey);

				$urlClass  = 'uc-hvr-underline-from-center';
				$listClass = '';
				if($sectionKey === $this->getActiveSection()){
					$urlClass  .= ' uc-hvr-underline-active';
					$listClass .= ' active-section';
				}

				$sectionListOutput  = "<li class=\"uc-grid-cell uc-grid-cell--autoSize $listClass\">";
				$sectionListOutput .= "<a class=\"$urlClass\" href=\"{$sectionUrl}\">";
				$sectionListOutput .= "<i class=\"fa $iconClass\" aria-hidden=\"true\"></i><span>{$sectionName}</span>";
				$sectionListOutput .= '</a>';
				$sectionListOutput .= '</li>';

				$arrNavBarArguments['sectionsListsOutput'] .= $sectionListOutput;

			}

		}

		return array_filter($arrNavBarArguments);

	}


	private static function getUserProfileFormFields()
	{
		$arrUserProfileFields = UserController::getProfiledUserProfileFormFields();

		if(empty($arrUserProfileFields))
			return array();

		$userProfileField = reset($arrUserProfileFields);
		if(! $userProfileField instanceof ProfileSectionField)
		{
			$profileSectionField           = new ProfileSectionField();
			$profileSectionField->Title    = __('My Profile', 'ultra-community');
			$profileSectionField->UniqueId = PHP_INT_MAX;
			array_unshift($arrUserProfileFields, $profileSectionField);
		}

		return $arrUserProfileFields;
	}

}