<?php

use UltraCommunity\UltraCommHelper;
use UltraCommunity\Controllers\UserController;
use UltraCommunity\Controllers\FrontPageController;
use UltraCommunity\UltraCommHooks;

defined('ABSPATH')  || exit;

$userPhotoUrl    = UltraCommHelper::getUserAvatarUrl(UserController::getProfiledUser());
$userDisplayName = UltraCommHelper::getUserDisplayName(UserController::getProfiledUser());
$userProfileUrl  = UltraCommHelper::getUserProfileUrl(UserController::getProfiledUser());
$userLogOutUrl   = FrontPageController::getLogOutPageUrl();

$sideBarProfileSettingsItems = isset($sideBarProfileSettingsItems) ? (array)$sideBarProfileSettingsItems : array();
$sideBarAccountSettingsItems = isset($sideBarAccountSettingsItems) ? (array)$sideBarAccountSettingsItems : array();
$sideBarGroupsSettingsItems  = isset($sideBarGroupsSettingsItems)  ? (array)$sideBarGroupsSettingsItems  : array();
$sideBarUserRelationsItems   = isset($sideBarUserRelationsItems)   ? (array)$sideBarUserRelationsItems   : array();

$viewProfileText     = esc_html__('View Profile', 'ultra-community');
$logOutText          = esc_html__('Log out', 'ultra-community');

$groupsSettingsText      = esc_html__('Groups Settings', 'ultra-community');
$profileSettingsText     = esc_html__('Profile Settings', 'ultra-community');
$accountSettingsText     = esc_html__('Account Settings', 'ultra-community');
$relationsSettingsText   = esc_html__('Relations', 'ultra-community');

$userCardOutput = $profileSettingsOutput = $groupsSettingsOutput = $accountSettingsOutput = $userRelationsOutput = null;

$userCardOutput = <<<UserCardOutput

<div class="uc-grid uc-grid--full uc-grid--center  uc-panel uc-sidebar-panel uc-settings-sidebar-user-card">

	<h3 class="uc-grid-cell uc-grid uc-grid--center uc-panel-head"></h3>

	<div class="uc-grid-cell uc-grid   uc-grid--full uc-panel-content">
		<div class="uc-settings-sidebar-avatar-holder" style="background-image:url($userPhotoUrl);"></div>
		<h4>$userDisplayName</h4>
		<div>
			<a class="uc-button uc-button-left-icon" href="$userProfileUrl">
				<i class="fa fa-power-off"></i><span>$viewProfileText</span>
			</a>
		</div>
	</div>

	<div class="uc-grid-cell uc-panel-footer">

		<a class="uc-button uc-button-left-icon" href="$userLogOutUrl">
			<i class="fa fa-power-off"></i><span>$logOutText</span>
		</a>

	</div>
</div>


UserCardOutput;

if(!empty($sideBarProfileSettingsItems))
{
	$profileSettingsList = null;
	foreach ($sideBarProfileSettingsItems as $sideBarProfileSettingsItem)
	{
		$profileSettingsList .= '<li>';
		$profileSettingsList .= empty($sideBarProfileSettingsItem->IsActive) ? '<a class="uc-hvr-underline-from-center" href="' . $sideBarProfileSettingsItem->Url . '">' : '<a class="uc-hvr-underline-from-center uc-hvr-underline-active" href="' . $sideBarProfileSettingsItem->Url . '">';
		$profileSettingsList .= '<i class="fa '. $sideBarProfileSettingsItem->Icon .'"></i>';
		$profileSettingsList .= '<span>' . $sideBarProfileSettingsItem->Title . '</span>';
		$profileSettingsList .= '</a>';
		$profileSettingsList .= '</li>';
	}

	$profileSettingsOutput = <<<ProfileSettingsOutput

<div class="uc-grid uc-grid--full uc-grid--center uc-grid--flex-cells uc-panel uc-sidebar-panel uc-settings-sidebar-profile">

	<h3 class="uc-grid-cell uc-grid uc-grid--center uc-grid--fit uc-grid--flex-cells uc-panel-head">
		<span class="uc-grid-cell uc-grid--center uc-grid--justify-center uc-grid-cell--autoSize"><i class="fa fa-cogs"></i></span>
		<span class="uc-grid-cell">$profileSettingsText</span>
	</h3>

	<div class="uc-grid-cell uc-panel-content">
		<ul>$profileSettingsList</ul>
	</div>

	<div class="uc-grid-cell uc-panel-footer"></div>

</div>

ProfileSettingsOutput;

}


if(!empty($sideBarGroupsSettingsItems))
{
	$groupsSettingsList = null;
	foreach ($sideBarGroupsSettingsItems as $sideBarGroupsSettingsItem)
	{
		$groupsSettingsList .= '<li>';
		$groupsSettingsList .= empty($sideBarGroupsSettingsItem->IsActive) ? '<a class="uc-hvr-underline-from-center" href="' . $sideBarGroupsSettingsItem->Url . '">' : '<a class="uc-hvr-underline-from-center uc-hvr-underline-active" href="' . $sideBarGroupsSettingsItem->Url . '">';
		$groupsSettingsList .= '<i class="fa '. $sideBarGroupsSettingsItem->Icon .'"></i>';
		$groupsSettingsList .= '<span>' . $sideBarGroupsSettingsItem->Title . '</span>';
		$groupsSettingsList .= '</a>';
		$groupsSettingsList .= '</li>';
	}

	$groupsSettingsOutput = <<<GroupsSettingsOutput

<div class="uc-grid uc-grid--full uc-grid--center uc-grid--flex-cells uc-panel uc-sidebar-panel uc-settings-sidebar-profile">

	<h3 class="uc-grid-cell uc-grid uc-grid--center uc-grid--fit uc-grid--flex-cells uc-panel-head">
		<span class="uc-grid-cell uc-grid--center uc-grid--justify-center uc-grid-cell--autoSize"><i class="fa fa-cogs"></i></span>
		<span class="uc-grid-cell">$groupsSettingsText</span>
	</h3>

	<div class="uc-grid-cell uc-panel-content">
		<ul>$groupsSettingsList</ul>
	</div>

	<div class="uc-grid-cell uc-panel-footer"></div>

</div>

GroupsSettingsOutput;

}

if(!empty($sideBarUserRelationsItems))
{
	$userRelationsList = null;
	foreach ($sideBarUserRelationsItems as $userRelationItem)
	{
		$userRelationsList .= '<li>';
		$userRelationsList .= empty($userRelationItem->IsActive) ? '<a class="uc-hvr-underline-from-center" href="' . $userRelationItem->Url . '">' : '<a class="uc-hvr-underline-from-center uc-hvr-underline-active" href="' . $userRelationItem->Url . '">';
		$userRelationsList .= '<i class="fa '. $userRelationItem->Icon .'"></i>';
		$userRelationsList .= '<span>' . $userRelationItem->Title . '</span>';
		$userRelationsList .= '</a>';
		$userRelationsList .= '</li>';
	}

	$userRelationsOutput = <<<UserRelationsOutput

<div class="uc-grid uc-grid--full uc-grid--center uc-grid--flex-cells uc-panel uc-sidebar-panel uc-settings-sidebar-profile">

	<h3 class="uc-grid-cell uc-grid uc-grid--center uc-grid--fit uc-grid--flex-cells uc-panel-head">
		<span class="uc-grid-cell uc-grid--center uc-grid--justify-center uc-grid-cell--autoSize"><i class="fa fa-cogs"></i></span>
		<span class="uc-grid-cell">$relationsSettingsText</span>
	</h3>

	<div class="uc-grid-cell uc-panel-content">
		<ul>$userRelationsList</ul>
	</div>

	<div class="uc-grid-cell uc-panel-footer"></div>

</div>

UserRelationsOutput;

}


if(!empty($sideBarAccountSettingsItems))
{
	$accountSettingsList = null;
	foreach ($sideBarAccountSettingsItems as $sideBarAccountSettingsItem)
	{
		$accountSettingsList .= '<li>';
		$accountSettingsList .=  empty($sideBarAccountSettingsItem->IsActive) ? '<a class="uc-hvr-underline-from-center" href="' . $sideBarAccountSettingsItem->Url . '">' : '<a class="uc-hvr-underline-from-center uc-hvr-underline-active" href="' . $sideBarAccountSettingsItem->Url . '">';
		$accountSettingsList .= '<i class="fa '. $sideBarAccountSettingsItem->Icon .'"></i>';
		$accountSettingsList .= '<span>' . $sideBarAccountSettingsItem->Title . '</span>';
		$accountSettingsList .= '</a>';
		$accountSettingsList .= '</li>';
	}

	$accountSettingsOutput = <<<AccountSettingsOutput

<div class="uc-grid uc-grid--full uc-grid--center uc-grid--flex-cells uc-panel uc-sidebar-panel uc-settings-sidebar-profile">

	<h3 class="uc-grid-cell uc-grid uc-grid--center uc-grid--fit uc-grid--flex-cells uc-panel-head">
		<span class="uc-grid-cell uc-grid--center uc-grid--justify-center uc-grid-cell--autoSize"><i class="fa fa-cogs"></i></span>
		<span class="uc-grid-cell">$accountSettingsText</span>
	</h3>

	<div class="uc-grid-cell uc-panel-content">
		<ul>$accountSettingsList</ul>
	</div>

	<div class="uc-grid-cell uc-panel-footer"></div>

</div>

AccountSettingsOutput;

}

echo $userCardOutput;

do_action(UltraCommHooks::ACTION_USER_PROFILE_SETTINGS_SIDEBAR_BEFORE_PROFILE_SECTIONS, UserController::getProfiledUser());
	echo $profileSettingsOutput;
do_action(UltraCommHooks::ACTION_USER_PROFILE_SETTINGS_SIDEBAR_AFTER_PROFILE_SECTIONS, UserController::getProfiledUser());

echo $groupsSettingsOutput, $userRelationsOutput, $accountSettingsOutput;

